/* -*-c++-*- 
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Cedric Pinson <cpinson@mekensleep.org>
 *
 */

//----------------------------------------------------------------------------//
// Includes                                                                   //
//----------------------------------------------------------------------------//

#include "max2exg.h"
#include <string>
#include <direct.h>

static bool showPrompts;
static bool exportSelected;
HINSTANCE hInstance;
int controlsInit = FALSE;

//----------------------------------------------------------------------------//
// Debug                                                                      //
//----------------------------------------------------------------------------//

#if 0
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#endif
//----------------------------------------------------------------------------//
// Constructors                                                               //
//----------------------------------------------------------------------------//

Max2Exg::Max2Exg()
{
}

//----------------------------------------------------------------------------//
// Destructor                                                                 //
//----------------------------------------------------------------------------//

Max2Exg::~Max2Exg()
{
}


BOOL WINAPI DllMain(HINSTANCE hinstDLL,ULONG fdwReason,LPVOID lpvReserved) 
{
  hInstance = hinstDLL;

  // Initialize the custom controls. This should be done only once.
  if (!controlsInit) {
    controlsInit = TRUE;
    InitCustomControls(hInstance);
    InitCommonControls();
  }
	
  return (TRUE);
}



//----------------------------------------------------------------------------//
// Following methods have to be implemented to make it a valid plugin         //
//----------------------------------------------------------------------------//

const TCHAR *Max2Exg::AuthorName()
{
  return _T("Cedric Pinson");
}

const TCHAR *Max2Exg::CopyrightMessage()
{
  return _T("Copyright (C) 2004 Mekensleep");
}

BOOL Max2Exg::SupportsOptions(int ext, DWORD options) {
  //assert(ext>1 || ext<0);
  return (options == SCENE_EXPORT_SELECTED) ? TRUE : FALSE;
}

int Max2Exg::DoExport(const TCHAR *name, ExpInterface *ei, Interface *i, BOOL suppressPrompts, DWORD options)
{
  mInterface=i;
  mRotateInRighthand=true;

  // Set a global prompt display switch
  showPrompts = suppressPrompts ? FALSE : TRUE;
  exportSelected = (options & SCENE_EXPORT_SELECTED) ? TRUE : FALSE;


  SetIncludeMesh(true);

  if(showPrompts) {
#if 0
    // Prompt the user with our dialogbox, and get all the options.
    if (!DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_EXGEXPORT_DLG),
			mInterface->GetMAXHWnd(), ExportDlgProc, (LPARAM)this)) {
      return 1;
    }
#endif
  }

  mNbTransforms=0;
  mNbMeshs=0;
  mNbLights=0;
  mNbCameras=0;


  mNameOfFile=std::string(name);
  std::string nameDebug=mNameOfFile+".log.txt";
  mStream=fopen(nameDebug.c_str(),"wt");
  if (!mStream)
    return 0;

  // fix me
  mSceneName=mNameOfFile.substr(0,mNameOfFile.length()-5);
  std::string::size_type it=mSceneName.rfind("\\");
  mSceneName=mSceneName.substr(it+1);



  exg::PyNodeGroup nd(0);
  if (IsSceneExport()) {
    nd=CreateRootNode(); // create root node and create a directory with name of scene
    mkdir(mSceneName.c_str());
  }

  BOOL r=ProcessNode(mSceneRoot,mInterface->GetRootNode());


  if (IsSceneExport()) {
    SetLightList(); // set list of light
    exg::PySaveScene(exg::PyNode(mSceneRoot.object),mNameOfFile);
    fprintf(mStream,"Scene %s Meshs %d Cameras %d Lights %d Transforms %d\n",mSceneName.c_str(),mNbMeshs,mNbCameras,mNbLights,mNbTransforms);
  }

  fclose(mStream);
  return r;
}


bool Max2Exg::IsMeshExport()
{
  if (mNameOfFile.find(".EMSH")!=std::string::npos || mNameOfFile.find(".emsh")!=std::string::npos)
    return true;
  return false;
}

bool Max2Exg::IsSceneExport()
{
  if (mNameOfFile.find(".ESCN")!=std::string::npos || mNameOfFile.find(".escn")!=std::string::npos)
    return true;
  return false;
}

#if 0
char toto[2000];
std::string nameOfMesh;
char toto2[2000];
#endif

bool Max2Exg::ProcessNode(exg::PyNodeGroup pyParent,INode* node) 
{	

  // Only export if exporting everything or it's selected
  int ns=node->Selected();
  if(!exportSelected || ns) {
     
    // The ObjectState is a 'thing' that flows down the pipeline containing
    // all information about the object. By calling EvalWorldState() we tell
    // max to eveluate the object at end of the pipeline.
    ObjectState os = node->EvalWorldState(0); 
    PropertyNode prop;

    // The obj member of ObjectState is the actual object we will export.
    if (os.obj) {
	
      // We look at the super class ID to determine the type of the object.
      GetNodeProperty(node,prop);
      switch(os.obj->SuperClassID()) {
      case GEOMOBJECT_CLASS_ID:
        {
          if (os.obj->ClassID() == Class_ID(TARGET_CLASS_ID, 0))
            break;

          std::string nameOfMesh=mNameOfFile;

          if (IsSceneExport()) {
            exg::PyNodeTransform tr=CreateNodeTransform(node);
            if (prop.ExportTransform()) {
              pyParent.AddNode(tr);
              pyParent.object=tr.object;
            }

            // ok export item
            if (prop.ExportItem()) {
              if (!prop.ExportItemSeparately()) {
                exg::PyNodeTransform pivot(0);
                exg::PyNodeMesh msh(0);
                CreateNodeMesh(node,msh,pivot);
                prop.Apply(msh);
                if (pivot.object) // pivot has mesh as child if not identity
                  tr.AddNode(pivot);
                else
                  tr.AddNode(msh);
                msh.GetName(nameOfMesh);
              } else {// special case we want to export mesh only but with its own local scene
                CreateDummySceneWithMesh(node,std::string(node->GetName()));
                CreateNameOfMesh(std::string(node->GetName()),nameOfMesh);
              }
            }

            prop.Apply(tr); // we apply attrib on the node after because we insert a null transform for billboard
            // between pivot and transform or between mesh and transform if no pivot
          }

          if (prop.ExportItem()) {
            if (IsMeshExport() || IsSceneExport())
              if (mMeshProcessed.find(nameOfMesh)==mMeshProcessed.end()) {
		            ExportGeomObject(node,mSceneName+"/"+nameOfMesh+".emsh");
                mMeshProcessed[nameOfMesh]=1;
              } else {
                fprintf(mStream,"Mesh %s already saved\n",nameOfMesh.c_str());
                mMeshProcessed[nameOfMesh]++;
              }
          }
        }
	break;
      case LIGHT_CLASS_ID:

        if (prop.ExportTransform() && prop.ExportItem()) {
          if (IsSceneExport()) {
            exg::PyNodeLight tr=CreateNodeLight(node);
            if (tr.object)
              pyParent.AddNode(tr);
            //mCurrentNode=tr;
          }
        }
        break;
      case CAMERA_CLASS_ID:

        if (prop.ExportTransform() && prop.ExportItem()) {
          if (IsSceneExport()) {
 exg::PyNodeCamera cm=CreateNodeCamera(node);
   pyParent.AddNode(cm);
     }
				 }
							   break;
							     
							     case HELPER_CLASS_ID:
							       if (prop.ExportTransform()) {
								 if (IsSceneExport()) {
 exg::PyNodeTransform cm=CreateNodeHelper(node);
   prop.Apply(cm);
   pyParent.AddNode(cm);
     pyParent=cm;
       }
											}
											     break;
											       }
    }
  }	
    
  // For each child of this node, we recurse into ourselves 
  // until no more children are found.
  for (int c = 0; c < node->NumberOfChildren(); c++) {
    if (!ProcessNode(pyParent,node->GetChildNode(c)))
      return false;
  }
    
  return true;
}




const TCHAR *Max2Exg::Ext(int i)
{
  switch(i)
    {
    case 0:
      return _T("emsh");
    case 1:
      return _T("escn");
    default:
      return _T("");
    }
}

int Max2Exg::ExtCount()
{
  return 2;
}

const TCHAR *Max2Exg::LongDesc()
{
  return _T("exg exporter (emsh,escn,eskl,eanm)");
}

const TCHAR *Max2Exg::OtherMessage1()
{
  return _T("");
}

const TCHAR *Max2Exg::OtherMessage2()
{
  return _T("");
}

const TCHAR *Max2Exg::ShortDesc()
{
  return _T("exg exporter");
}

void Max2Exg::ShowAbout(HWND hWnd)
{
}

unsigned int Max2Exg::Version()
{
  return 100;
}
