/* -*-c++-*- 
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Cedric Pinson <cpinson@freesheep.org>
 *
 */

#ifndef EXG_SCENE_BIND_H
#define EXG_SCENE_BIND_H

#include <exg/exg_matrix4.h>
#include <exg/exg_matrix3.h>
#include <exg/exg_vector3.h>
#include <exg/exg_vector4.h>
#include <string>
#include <vector>
#include <Python.h>


class PyNode;

PyObject* PyNewMatrix(const exg::Matrix3f& m);
PyObject* PyNewMatrix(const exg::Matrix4f& m);
PyObject* PyNewVector(const exg::Vector3f& vec);
PyObject* PyNewVector(const exg::Vector4f& vec);
PyObject* PyGetClass(const std::string& classname);
PyObject* PyNewObject(const std::string& classname, const std::string& instancename="o"); 
bool PyIsInstance(PyObject* o,const std::string& s);
void PySaveScene(PyNode* node, const std::string& name);

exg::Vector4f PyGetVector(PyObject* o);
exg::Matrix4f PyGetMatrix(PyObject* o);


class PyBase
{
public:
  PyBase(){}
  PyBase(const std::string& cname) {
    object = PyNewObject(cname); 
  }

  PyObject* object;
};



class PyNode : public PyBase
{

 public:

  PyNode() : PyBase("Node") {}
  PyNode(const std::string& cname) : PyBase(cname) {}
  PyNode(PyObject* o) { object=o;}

  void SetName(const std::string& n);
  std::string GetName();
};


class PyNodeGroup : public PyNode
{
  std::vector<PyNode*> mNodes;

 public:
  PyNodeGroup(): PyNode("NodeGroup"){ }
  PyNodeGroup(const std::string& n): PyNode(n){}

  void AddNode(PyNode* _o);
  PyNode* GetNode(int i);

};


class PyNodeTransform : public PyNodeGroup
{
 public:
  PyNodeTransform():PyNodeGroup("NodeTransform"){}
  PyNodeTransform(const std::string& n):PyNodeGroup(n){}

  void SetMatrix(const exg::Matrix4f&);
  exg::Matrix4f GetMatrix();
};

class PyNodeMesh : public PyNode
{
  std::string filename; // remove it should call python part to be synced

public:
  PyNodeMesh():PyNode("NodeMesh"){ }

  void SetFilename(const std::string& n);
  std::string GetFilename();
};

class PyNodeCamera : public PyNode
{
public:
  PyNodeCamera():PyNode("NodeCamera"){ }

  void SetMatrix(const exg::Matrix4f& m);
  void SetFov(float fov);
  void SetNear(float n);
  void SetFar(float f);

  exg::Matrix4f GetMatrix();
  float GetFov();
  float GetNear();
  float GetFar();
};


class PyNodeLight : public PyNodeTransform
{
 public:
  PyNodeLight():PyNodeTransform("NodeLight"){}
  PyNodeLight(const std::string& n):PyNodeTransform(n){}

  void SetColor(const exg::Vector4f& c);
  void SetColor(const exg::Vector3f& c) { SetColor(exg::Vector4f(c,1));}
  exg::Vector4f GetColor();
};


class PyNodeDirectionalLight : public PyNodeLight
{
 public:
   PyNodeDirectionalLight():PyNodeLight("NodeDirectionLight"){}

};

class PyNodePointLight : public PyNodeLight
{
 public:
   PyNodePointLight():PyNodeLight("NodePointLight"){}

};

class PyNodeSpotLight : public PyNodeLight
{
 public:
   PyNodeSpotLight():PyNodeLight("NodeSpotLight"){}

};

class PyNodeAmbientLight : public PyNodeLight
{
 public:
   PyNodeAmbientLight():PyNodeLight("NodeAmbientLight"){}

};

#endif
