/*
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Loic Dachary <loic@gnu.org>
 *  Cedric Pinson <cpinson@freesheep.org>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <cassert>
#include <iostream>

#include <osg/Node>
#include <osg/Group>
#include <osg/MatrixTransform>
#include <osg/Geode>
#include <osg/NodeVisitor>
#include <osg/Geometry>
#include <osg/Material>
#include <osg/Texture2D>
#include <osg/StateSet>
#include <osg/Light>
#include <osg/LightSource>
#include <osg/LightModel>
#include <osgDB/ReadFile>

#include <exg/exg_visitor.h>
#include <exg/exg_mesh.h>
#include <exg/exg_python_scene.h>

#include <exg_osg.h>
#include <vector>
#include <string>

namespace exg
{


  class ESCN2OSG
  {

    int mCurrentLight;

  public:

    ESCN2OSG() { mCurrentLight=0; }

    void SetArgs(const std::vector<std::string>& args) {
      mArguments=args;
//       for (int i=0;i<mArguments.size();i++)
// 	std::cout << mArguments[i] << std::endl;

      // search where source data come from
      for (int i=0;i<mArguments.size();i++)
	if (mArguments[i].find(".escn")!=std::string::npos) {
	  std::string::size_type end=mArguments[i].rfind("/");
	  if (end!=std::string::npos) {
	    mSourceData=mArguments[i].substr(0,end+1);
	    std::cout << "Data directory " << mSourceData << std::endl;
	    break;
	  }
	}
    }

    osg::Node* Convert(PyObject* pyobject) {
      osg::MatrixTransform* transform = new osg::MatrixTransform;
      //      osg::Matrix mt2=osg::Matrix::rotate(M_PI/2,osg::Vec3(-1,0,0));
      osg::Matrix mt2=osg::Matrix::identity();
      transform->setMatrix(mt2);
      mRoot=transform;
      return Convert(pyobject, transform);
    }

    osg::Node* ESCN2OSG::Convert(PyObject* pyobject, osg::Group* parent);

  private:
    osg::Node* mRoot;
    std::vector<std::string> mArguments;
    std::string mSourceData;

  };



  osg::Node* PyESCN2OSG(PyObject* pyobject,std::vector<std::string> args) {
    ESCN2OSG converter;
    converter.SetArgs(args);
    return converter.Convert(pyobject);
  }





  osg::Node* ESCN2OSG::Convert(PyObject* pyobject, osg::Group* parent) {
    osg::Node* node = 0;
    std::string nodeType="unknown";

    if (!pyobject)
      return 0;

    PyNode nn(pyobject);
    std::cerr << nn.GetName() << std::endl;


    if(exg::PyIsInstance(pyobject, "NodeGroup")) {
      osg::Group* group = 0;
      nodeType="NodeGroup";

      if(exg::PyIsInstance(pyobject, "NodeTransform")) {
	nodeType="NodeTransform";
	osg::MatrixTransform* transform=0;

	if(exg::PyIsInstance(pyobject, "NodeLight")) {


	  bool ambient = exg::PyIsInstance(pyobject, "NodeAmbientLight");
	  if (!ambient) {

	    nodeType="NodeLight";
	    osg::Light* light = new osg::Light;
	    light->setLightNum(mCurrentLight++);
	    if (mCurrentLight==7) // loop on last ligth if more than 8
	      mCurrentLight=6;

	    exg::PyNodeLight gl(pyobject);
	    exg::Vector4f df=gl.GetColor();
	    osg::Vec4 lld(df[0],df[1],df[2],df[3]);
	  
	    light->setDiffuse(lld);


	    bool directional = exg::PyIsInstance(pyobject, "NodeDirectionalLight");
	    if(directional) {
	      nodeType="NodeDirectionalLight";
	      exg::PyNodeDirectionalLight l(pyobject);
	      exg::Vector4f ltmp=l.GetMatrix()[2];
	      osg::Vec3 ld(ltmp[0],ltmp[1],ltmp[2]);
	      light->setDirection(ld);
	    }

	    bool spot = exg::PyIsInstance(pyobject, "NodeSpotLight");
	    if(spot) {
	      nodeType="NodeSpotLight";
	      exg::PyNodeSpotLight l(pyobject);
	      light->setSpotCutoff(l.GetCutoff());
	      exg::Vector4f ltmp=l.GetMatrix()[2];
	      osg::Vec3 ld(ltmp[0],ltmp[1],ltmp[2]);
	      light->setDirection(ld);

	      ltmp=exg::Vector4f(l.GetMatrix()[0][3],l.GetMatrix()[1][3],l.GetMatrix()[2][3],1);
	      osg::Vec4 ld2(ltmp[0],ltmp[1],ltmp[2],1);
	      light->setPosition(ld2);
	    }

	    bool point = exg::PyIsInstance(pyobject, "NodePointLight");
	    if(point) {
	      nodeType="NodePointLight";
	      exg::PyNodePointLight l(pyobject);
	      exg::Vector4f ltmp=exg::Vector4f(l.GetMatrix()[0][3],l.GetMatrix()[1][3],l.GetMatrix()[2][3],1);
	      osg::Vec4 ld(ltmp[0],ltmp[1],ltmp[2],1);
	      light->setPosition(ld);
	    }

	    osg::LightSource* light_source = new osg::LightSource;
	    light_source->setLight(light);
	    light_source->setLocalStateSetModes(osg::StateAttribute::ON);
	    light_source->getOrCreateStateSet()->setMode(GL_CULL_FACE,osg::StateAttribute::ON);
	    node = light_source;
	    group=light_source;
	  } else {

	    osg::LightModel* lm=new osg::LightModel;
	    osg::StateSet* sets=mRoot->getOrCreateStateSet();

	    exg::PyNodeLight gl(pyobject);
	    exg::Vector4f df=gl.GetColor();
	    osg::Vec4 lld(df[0],df[1],df[2],df[3]);
	    
	    lm->setAmbientIntensity(lld);
	    sets->setAttributeAndModes(lm);
	    node = 0;
	    group=0;

	  }
	} else {
	  transform = new osg::MatrixTransform;
	  exg::PyNodeTransform t(pyobject);
	  osg::Matrix mt2=osg::Matrix::rotate(M_PI/2,osg::Vec3(-1,0,0));
	  exg::Matrix4f m=t.GetMatrix();
	  osg::Matrix mt(m[0][0],m[1][0],m[2][0],m[3][0],
			 m[0][1],m[1][1],m[2][1],m[3][1],
			 m[0][2],m[1][2],m[2][2],m[3][2],
			 m[0][3],m[1][3],m[2][3],m[3][3]);
	  //mt=mt2*mt;
	  //  mt=osg::Matrix::translate(osg::Vec3(0,0,0));
	  // for (int i=0;i<4;i++)
	  //  std::cout << m[i][0] << " " << m[i][1] << " " << m[i][2]<< " " << m[i][3] <<std::endl;

	  transform->setMatrix(mt);
	  group = transform;
	}

      } else {
	group = new osg::Group;
      }

      exg::PyNodeGroup grp(pyobject);
      int nbSubs=grp.GetNbNodes();
      for(int i = 0; i < nbSubs; i++)
	Convert(grp.GetNode(i).object, group);

      node = group;

    } else if(exg::PyIsInstance(pyobject, "NodeMesh")) {
      nodeType="NodeMesh";
      exg::PyNodeMesh msh(pyobject);
      std::string filename;
      msh.GetFilename(filename);
      Object* object=0;
      node=0;
      if (!filename.empty()) {
	
	filename=mSourceData + filename;
	std::ifstream fs(filename.c_str());
	if (fs) {
	  std::cout << "mesh " << filename <<std::endl;
	  exg::Pointer<exg::Mesh> mesh=0;
	  exg::Load(mesh,fs);
	  object=mesh.Get();
	  node = PyEMSH2OSG(object,mArguments);
	  node->getOrCreateStateSet()->setMode(GL_CULL_FACE,osg::StateAttribute::ON);

	} else {
	  std::cout << "mesh file not found " << filename <<std::endl;
	}
      } 
    }

    if(node) {
      exg::PyNode nm(pyobject);
      node->setName(nm.GetName());
      if(parent) parent->addChild(node);
    }
    return node;
  }


};
