#
#
# Copyright (C) 2004 Mekensleep
#
#	Mekensleep
#	24 rue vieille du temple
#	75004 Paris
#       licensing@mekensleep.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#  Cedric Pinson <cpinson@freesheep.org>
#  Loic Dachary <loic@gnu.org>
#
#
from xml.marshal import *


def save(scene, filename):
    print "Saving to %s" % filename
    m = generic.Marshaller()
    m.dump(scene,open(filename,'w'))

def load(filename):
    m=generic.Unmarshaller()
    return m.load(open(filename,'r'))


class Vector:
    def __init__(self, *args, **kwargs):
        if len(args)==1:
            self.data = args[0]
        elif len(args)==3:
            self.data=[args[0],args[1],args[2],1]
        elif len(args)==4:
            self.data=[args[0],args[1],args[2],args[3]]
        else:
            self.data=[0,0,0,1]

    def __str__(self):
        b=""
        for i in range(4):
            b=b+str(self.data[i]) + " "
        return b;

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key]=value

        
class Matrix:
    def __init__(self, *args, **kwargs):
        self.data = len(args) > 0 and args[0] or [Vector(1,0,0,0),Vector(0,1,0,0),Vector(0,0,1,0),Vector(0,0,0,1)]


    def getTranslation(self):
        return Vector(self.data[0][3],self.data[1][3],self.data[2][3],1.0)

    def setTranslation(self,t):
        self.data[0][3]=t[0]
        self.data[1][3]=t[1]
        self.data[2][3]=t[2]
        self.data[3][3]=1.0

    def __str__(self):
        b=""
        for i in range(4):
            b=b+str(self.data[i]) + "\n"
        return b;

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key]=value
    
class Node:
    def __init__(self, *args, **kwargs):
        self.name = len(args) > 0 and args[0]
        self.name = kwargs.has_key("name") and kwargs["name"] or None

    def getName(self):
        return self.name

    def setName(self, name):
        self.name = name

    def traverse(self,callback):
        callback(self)

    def apply(self, func):
        func(self)

class NodeGroup(Node):

    def __init__(self, *args, **kwargs):
        Node.__init__(self, *args, **kwargs)
        self.children = kwargs.has_key("children") and kwargs["children"] or []
    
    def addNode(self,node):
        self.children.append(node)        

    def removeNode(self,node):
        if node in self.children:
            self.children.remove(node)

    def getNodes(self):
        return self.children

    def traverse(self,callback):
        Node.traverse(self,callback)
        for node in self.children:
            node.traverse(callback)


class NodeTransform (NodeGroup):

    def __init__(self, *args, **kwargs):
        NodeGroup.__init__(self, *args, **kwargs)
        self.matrix = kwargs.has_key("matrix") and kwargs["matrix"] or Matrix()

    def setMatrix(self,tr):
        self.matrix=tr

    def getMatrix(self):
        return self.matrix
        
    def getTranslation(self):
        return self.matrix.getTranslation()

    def setTranslation(self,i):
        self.matrix.setTranslation(i)


class NodeMesh (Node):

    def __init__(self, *args, **kwargs):
        Node.__init__(self, *args, **kwargs)
        self.object = kwargs.has_key("object") and kwargs["object"] or None
        self.filename = kwargs.has_key("filename") and kwargs["filename"] or None
    
    def getObject(self):
        return self.object

    def setObject(self, object):
        self.object = object

    def setFilename(self, fn):
        self.filename=fn

    def getFilename(self):
        return self.filename



class NodeLight(NodeTransform):

    def __init__(self, *args, **kwargs):
        NodeTransform.__init__(self, *args, **kwargs)
        self.color = kwargs.has_key("color") and kwargs["color"] or Vector(0.5,0.5,0.5,1)
        self.include = 0

    def setColor(self,c):
        self.color=c

    def getColor(self):
        return self.color

    def getDirection(self):
        return Vector(matrix[2][0],matrix[2][1],matrix[2][2],1)

    def setDirection(self,d):
        self.matrix[2][0]=d[0]
        self.matrix[2][1]=d[1]
        self.matrix[2][2]=d[2]

    def setInclude(self):
        self.include=1

    def setExclude(self):
        self.include=0

class NodeAmbientLight(NodeLight):

    def __init__(self, *args, **kwargs):
        NodeLight.__init__(self, *args, **kwargs)

class NodeDirectionalLight(NodeLight):

    def __init__(self, *args, **kwargs):
        NodeLight.__init__(self, *args, **kwargs)
        self.matrix = kwargs.has_key("direction") and Matrix(Vector(1,0,0,0),Vector(0,1,0,0),kwargs["direction"],Vector(0,0,0,1)) or Matrix()
        

class NodePointLight(NodeLight):

    def __init__(self, *args, **kwargs):
        NodeLight.__init__(self, *args, **kwargs)
        self.radius = kwargs.has_key("radius") and kwargs["radius"] or 10
        self.matrix = kwargs.has_key("position") and Matrix(Vector(1,0,0,kwargs["position"][0]),Vector(1,0,0,kwargs["position"][1]),Vector(1,0,0,kwargs["position"][2]),Vector(1,0,0,1)) or Matrix()
    
    def getRadius(self):
        return self.radius

    def setRadius(self,d):
        self.radius=d

class NodeCamera(Node):

    def __init__(self, *args, **kwargs):
        Node.__init__(self, *args, **kwargs)
        self.matrix = kwargs.has_key("matrix") and kwargs["matrix"] or Matrix()
        self.fov = kwargs.has_key("fov") and kwargs["fov"] or 60.0
        self.near= kwargs.has_key("near") and kwargs["near"] or 0.5
        self.far= kwargs.has_key("far") and kwargs["far"] or 10000
        self.target= kwargs.has_key("target") and kwargs["target"] or Vector()
    
    def setMatrix(self,tr):
        self.matrix=tr

    def setNear(self,n):
        self.near=n

    def setFar(self,f):
        self.far=f

    def setFov(self,f):
        self.fov=f

    def getMatrix(self):
        return self.matrix
        
    def getNear(self):
        return self.near

    def getFar(self):
        return self.far

    def getFov(self):
        return self.fov

    def getTarget(self):
        return self.target

    def setTarget(self,t):
        self.target=t
        

class NodeSpotLight(NodePointLight):

    def __init__(self, *args, **kwargs):
        NodePointLight.__init__(self, *args, **kwargs)
        self.cutoff = kwargs.has_key("cutoff") and kwargs["cutoff"] or 90
    
    def getCutoff(self):
        return self.cutoff

    def setCutoff(self,d):
        self.cutoff=d


def test():
    a=NodeGroup()
    a.setName("nodeGroup")
    b=NodeTransform()
    b.setName("transform node")
    b.setTranslation(Vector(1,2,3))
    a.addNode(b)
    c=NodeMesh()
    c.setName("mesh.emsh")
    b.addNode(c)

    d=NodeGroup()
    d.setName("light group")
    b.addNode(d)


    l0=NodeAmbientLight()
    l0.setColor((0.5,0,0))
    d.addNode(l0)

    l1=NodeDirectionalLight()
    l1.setColor((0,0,1))
    l1.setDirection(Vector(0,0,1))
    d.addNode(l1)

    l2=NodePointLight()
    l2.setColor((0,1,0))
    l2.setTranslation(Vector(50,50,50))
    l2.setRadius(200)
    d.addNode(l2)

    l3=NodeSpotLight()
    l3.setColor((0,1,0))
    l3.setTranslation(Vector(50,50,50))
    l3.setRadius(200)
    l3.setDirection(Vector(1,0,0))
    l3.setCutoff(60)
    d.addNode(l3)

    save(a,"test.escn")
    z=load("test.escn")
    print z
    
