/*
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Loic Dachary <loic@gnu.org>
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <cassert>
#include <sstream>

#include <exg/exg_basetypes.h>

int main(int argc, char** argv) {
  // create access VectorInt
  exg::VectorInt* vi = new exg::VectorInt(5);
  vi->push_back(3);
  assert((*vi)[5] == 3);

  // create access VectorFloat
  exg::VectorFloat* vf = new exg::VectorFloat(5);
  vf->push_back(3.2f);
  assert((*vf)[5] == 3.2f);

  // create access VectorString
  exg::VectorString* vs = new exg::VectorString(5);
  vs->push_back("tchou");
  assert((*vs)[5] == "tchou");

  // Save / restore VectorInt
  {
    std::stringstream buffer(std::ios_base::out|std::ios_base::in);
    buffer.exceptions();

    exg::Save(*vi, buffer);
    std::string reference = buffer.str();
    std::cerr.write(buffer.str().c_str(), buffer.str().size()) << std::endl;

    exg::Pointer<exg::Object> tmp;
    buffer.seekp(0, std::ios_base::beg);
    exg::Load(tmp, buffer);
    exg::VectorInt* vi2 = tmp->AsVectorInt();
    assert(reference == buffer.str());

    assert((*vi2)[5] == (*vi)[5]);
  }

  // Save / restore VectorFloat
  {
    std::stringstream buffer(std::ios_base::out|std::ios_base::in);
    buffer.exceptions();

    exg::Save(*vf, buffer);
    std::string reference = buffer.str();
    std::cerr.write(buffer.str().c_str(), buffer.str().size()) << std::endl;

    exg::Pointer<exg::Object> tmp;
    buffer.seekp(0, std::ios_base::beg);
    exg::Load(tmp, buffer);
    exg::VectorFloat* vf2 = tmp->AsVectorFloat();
    assert(reference == buffer.str());

    assert((*vf2)[5] == (*vf)[5]);
  }

  // Save / restore VectorString
  {
    std::stringstream buffer(std::ios_base::out|std::ios_base::in);
    buffer.exceptions();

    exg::Save(*vs, buffer);
    std::string reference = buffer.str();
    std::cerr.write(buffer.str().c_str(), buffer.str().size()) << std::endl;

    exg::Pointer<exg::Object> tmp;
    buffer.seekp(0, std::ios_base::beg);
    exg::Load(tmp, buffer);
    exg::VectorString* vs2 = tmp->AsVectorString();
    assert(reference == buffer.str());

    assert((*vs2)[5] == (*vs)[5]);
  }

  // Create heterogeneous vector
  exg::VectorObjectPointer* vv = new exg::VectorObjectPointer;
  vv->push_back(vi);
  vv->push_back(vf);
  vv->push_back(vs);
  vv->push_back(vv);

  // Save / restore heterogeneous with circular references
  {
    std::stringstream buffer(std::ios_base::out|std::ios_base::in);
    buffer.exceptions();

    exg::Save(*vv, buffer);
    std::string reference = buffer.str();
    std::cerr.write(buffer.str().c_str(), buffer.str().size()) << std::endl;

    exg::Pointer<exg::Object> tmp = 0;
    buffer.seekp(0, std::ios_base::beg);
    exg::Load(tmp, buffer);
    assert(reference == buffer.str());
  }

  // Create heterogeneous map
  exg::MapObjectPointer* m = new exg::MapObjectPointer;
  (*m)["one"] = vi;
  (*m)["two"] = vf;
  (*m)["three"] = vs;
  (*m)["four"] = vv;
  (*m)["five"] = m;
  
  // Save / restore heterogeneous with circular references
  {
    std::stringstream buffer(std::ios_base::out|std::ios_base::in);
    buffer.exceptions();

    exg::Save(m, buffer);
    std::string reference = buffer.str();
    std::cerr.write(buffer.str().c_str(), buffer.str().size()) << std::endl;

    exg::Pointer<exg::Object> tmp = 0;
    buffer.seekp(0, std::ios_base::beg);
    exg::Load(tmp, buffer);
    assert(reference == buffer.str());
  }
}
