/**
    Copyright (C) 2004 Cedric Pinson <cpinson@freesheep.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 ****************************************************************************
 * @file   exg_vertex.cxx
 *
 * @brief   base for vertex exchange
 *
 *****************************************************************************
 *
 * @author  Cedric Pinson
 *
 * @date    Created 2002/02
 *
 * @version $Id: exg_point.cxx,v 1.6 2004/04/28 10:37:19 loic Exp $
 *
 ****************************************************************************/

#include "exg/exg_point.h"
#include "exg/exg_polygon.h"
#include "exg/exg_vertex.h"
#include "exg/exg_material.h"
#include "exg/exg_mesh.h"
#include "exg/exg_vector3.h"
#include <exg/exg_object.h>
#include <exg/exg_visitor_utils.h>
#include <exg/exg_visitor_generate_normal.h>
#include <exg/exg_visitor_connectivity.h>
#include <iostream>
#include <fstream>
#include <vector>



#define NB_POINTS 100000*3
#define NB_POLYGONS 8

using namespace exg;

class MyVisitor : public Visitor
{

 public:

  MyVisitor():Visitor(ALL){}

  void Apply(Object& node) {
    Traverse(node);
  }

  void Apply(Material& node) {
    std::cout << "Apply Material " << &node << std::endl;
    Apply((Object&)node);
  }

  void Apply(Polygon& node) {
    std::cout << "Apply Polygon " << &node << std::endl;
    Apply((Object&)node); 
  }

  void Apply(Vertex& node) {
    std::cout << "Apply Vertex " << &node << std::endl;
    Apply((Object&)node); 
  }

  void Apply(Point& node) {
    std::cout << "Apply Point " << &node << " " 
              << node.GetPosition()[0] << " " 
              << node.GetPosition()[1] << " " 
              << node.GetPosition()[2] << std::endl;
    Apply((Object&)node); 
  }

  void Apply(Mesh& node) {
    std::cout << "Apply Mesh " << &node << std::endl;
    Apply((Object&)node); 
  }

  void Apply(File& node) {
    std::cout << "Apply File " << &node << std::endl;
    Apply((Object&)node); 
  }
  

};


int main(int argc,char** argv)
{


  Pointer<Mesh> mesh=new Mesh;
  mesh->SetName("foo");
  mesh->SetNbFiles(2);
  mesh->SetNbMaterials(2);
  mesh->GetFile(0)->SetName("bar.tga");
  mesh->GetMaterial(0)->SetName("material_foo");
  mesh->SetNbPoints(NB_POLYGONS*3);
  mesh->SetNbPolygons(NB_POLYGONS);
  

  Vector3f color(0,1,2);
  (*mesh->GetMaterial(0))["color_diffuse"]=new VectorFloat(color);

  for (int i=0;i<NB_POLYGONS*3;i++)
    mesh->GetPoint(i)->SetPosition(Vector3f(i,i,i));


  for (int i=0;i<NB_POLYGONS;i++) {
    mesh->GetPolygon(i)->SetNbVertexes(3);
    mesh->GetPolygon(i)->SetMaterial(mesh->GetMaterial(0));
    for (int j=0;j<3;j++) {
      mesh->GetPolygon(i)->GetVertex(j)->SetPoint(mesh->GetPoint(i*3+j));
      (*mesh->GetPolygon(i)->GetVertex(j))["normal"]=new VectorFloat(Vector3f(0,1,0));
    }
  }


  std::cout << mesh->GetName() << std::endl;
  std::cout << mesh->GetFile(0)->GetName() << std::endl;


  MyVisitor vtest;
  mesh.Get()->Accept(vtest);


  VisitorBuildEdgeConnectivity ctest;
  mesh.Get()->Accept(ctest);

  VisitorGenerateNormalToMaterial ntest(&ctest);
  mesh.Get()->Accept(ntest);



  VisitorDump dtest;
  mesh.Get()->Accept(dtest);

  {
    std::ofstream out("a.emsh");
    Pointer<Object> o = mesh.Get();
    Save(o, out);
  }
  {
    Pointer<Object> other_mesh = new Mesh;
    std::ifstream in("a.emsh");
    Load(other_mesh, in);
    other_mesh->operator<<(std::cerr);
  }

  Point* p=new Point;
  printf("addy 0x%08x\n", (int)(*p)["position"].Get());
  Vector3f b(0,1,2);
  p->SetPosition(b);
  Pointer<VectorFloat> a = new VectorFloat(p->GetPosition());
  std::cout << (*a)[0] << (*a)[1] << (*a)[2] << std::endl;
  printf("addy 0x%08x\n", (int)(*p)["position"].Get());

  return 0;
}
