/*
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Loic Dachary <loic@gnu.org>
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <cassert>
#include <iostream>

#include <exg/exg.h>

int main(int argc, char** argv) {
  exg::Pointer<exg::Mesh> meshp = new exg::Mesh;
  exg::Mesh& mesh = *(meshp.Get());

  float half_width = 10.1f;
  float half_height = 10.1f;
  float positions[] = {
    -half_width, -half_height, 0.f,
    half_width, -half_height, 0.f,
    half_width, half_height, 0.f,
    -half_width, half_height, 0.f
  };

  exg::Polygon* polygon = new exg::Polygon;
  mesh["polygons"]->AsVectorObjectPointer()->push_back(polygon);

  exg::VectorObjectPointer* points = mesh["points"]->AsVectorObjectPointer();
  exg::VectorObjectPointer* vertexes = (*polygon)["vertexes"]->AsVectorObjectPointer();
  for(unsigned int i = 0; i < sizeof(positions) / sizeof(float); i += 3) {
    exg::Point* point = new exg::Point;
    (*point)["position"] = new exg::VectorFloat(positions + i, positions + i + 3);
    exg::Vertex* vertex = new exg::Vertex;
    vertex->SetPoint(point);

    vertexes->push_back(vertex);
    points->push_back(point);
  }

  exg::Material* material = new exg::Material;
  polygon->SetMaterial(material);

  float color[] = { 1.f, 0.f, 0.f };
  (*material)["diffuse"] = new exg::VectorFloat(color, color + 3);

  Save(mesh, std::cout);
}
